from setuptools import setup

setup (name = 'probability_dstn', 
    version='0.2',
    description= 'Gaussian distributions and binomial distribution',
    packages= ['probability_dstn'],
    author= 'Okeowo Tunde',
    author_email ='okeowosamuel58.to@gmail.com',
    zip_safe = False)


     